 
	  %MACRO calc_cox_iter_stats_central;

  /* Wrapper for calculating various genmod regression stats per iteration at the central site. */

   %PUT;
   %PUT -----   Start of the macro calc_cox_iter_stats_central  iter_nb=&iter_nb. ;
   %PUT;

   	 /* Defensive coding in case data partners use different case for vars in their input dataset.
			Although SAS is not case sensitive as far as variable names are concerned this affects 
			the case of the values in the _NAME_ variable of the sscp dataset.
		 */

   
	   PROC DATASETS  MEMTYPE=DATA NOLIST;
		   MODIFY &prefix.sscp_c;
		     ATTRIB _ALL_ LABEL=" ";
		     ATTRIB _ALL_ FORMAT=;
		RUN;
		QUIT;

	 DATA &prefix.sscp_c1;
		SET &prefix.sscp_c;
		   ARRAY var{*} &independent_vars. &dependent_vars. &strata_vars.;

			 DO i=1 TO DIM(var);
			   IF lowcase(_NAME_)=lowcase(VNAME(var{i})) THEN _NAME_ =VNAME(var{i});
			 END;
			 DROP i;
	 RUN;

	/* If stratification vars do not include DP identifier variable dp_cd then calculation of Hessian and 
	   gradient occurs at the central site. Otherwise the contributions to these quantities are 
	   calculated a each DP and brought to the central site.
  */

	 %IF &pos_dp_cd_in_strata_vars. =0 %THEN
	       %DO;

		   /*  Summarize contributions from all DPs	   */
		   
	PROC MEANS DATA=&prefix.sscp_c1(RENAME=(_TYPE_=_TYPE_1) 
																	WHERE=(_TYPE_1  NOT IN ("MEAN", "STD") )
	                                                               ) NWAY NOPRINT;
	   CLASS &strata_vars.  &dependent_vars. _TYPE_1	 _NAME_ /MISSING;
			VARS  Intercept &independent_vars. _WEIGHT_  _EVENT_;
	    OUTPUT 	 OUT=DPLocal.&prefix.sscp(TYPE=sscp DROP=_TYPE_ _FREQ_ RENAME=(_TYPE_1=_TYPE_))
						   SUM=;
	  RUN;

	  /* The main purpose of the macro below is to calculate Hessian matrix and and gradient.
		  After convergence is achieved it also calculates baseline hazard, overall summaries,
	      and means of all covariates by stratification variables.
	  */

		     %calc_cox_grad_hessian(tbl_cox_sscp_in=dplocal.&prefix.sscp
													    ,tbl_model_coeff_in =&infolder_lib..&prefix_prev.Model_Coeff
														,tbl_mean_vars_by_st_out=dplocal.&prefix0.mean_vars_by_st
														,tbl_resid_sum_out=dplocal.&prefix.resid_sum
														,tbl_baseln_hazard=dplocal.&prefix0.baseln_hazard
														,tbl_Gradient_out =dplocal.&prefix.Gradient
														,tbl_Hessian_out =dplocal.&prefix.Hessian
	                                                      ) ;

		 	/*If &exist_init_events_time_set=0 then create &tbl_events_time_set based on 
		data received from the data partners at the first iteration. */

				 %IF &iter_nb.=1 AND &exist_init_events_time_set.=0 %THEN
				                        %DO;
			
				PROC SORT DATA= DPLocal.&prefix.sscp(KEEP=&strata_vars. &dependent_vars. _NAME_ _TYPE_)
				                       OUT=infolder.&prefix0.all_events_time_set(KEEP=&strata_vars. &dependent_vars.) 
                                        NODUPKEY;
					BY 	&strata_vars. &dependent_vars. ;
						WHERE  LOWCASE(_NAME_)="intercept" AND _TYPE_="SSCP";
				RUN;

				 %LET tbl_events_time_set=&prefix0.all_events_time_set;

							             %END; /* End of do-group for &iter_nb.=1 AND &exist_init_events_time_set.=0*/

		    %END;  /* End of do-group for &pos_dp_cd_in_strata_vars. =0 */	

			   /* ELSE condition: When strata variables include dp_cd variable */
  %ELSE
           %DO;

		   %IF   &iter_nb. = 1  %THEN
 								%DO;
	  DATA  dplocal.&prefix0.mean_vars_by_st;
	     SET  &prefix0.mean_vars_by_st_c;
	 RUN;
								 %END;

     PROC SORT DATA=dplocal.&prefix0.mean_vars_by_st;
	  BY &strata_vars. ;
	 RUN;

	   %IF &iter_nb. = %eval(&iter_nb_at_conv. + 1) %THEN
 								%DO;
	  DATA  dplocal.&prefix0.baseln_hazard;
	     SET  &prefix0.baseln_hazard_c;
	 RUN;

	 PROC SORT DATA=dplocal.&prefix0.baseln_hazard;
	  BY &strata_vars. &dependent_vars.;
	 RUN;
								 %END;

	PROC MEANS DATA=&prefix.resid_sum_c NWAY NOPRINT;
	        VARS _L__Sum _L0__Sum  _LL__Sum;
	    OUTPUT 	 OUT=dplocal.&prefix.resid_sum(DROP=_TYPE_ _FREQ_ )
						   SUM=;
	  RUN;	

		 /*When strata variables include dp_cd variable	the sscp_c1 has contributions to
           Hessian and gradient from each data partner . We will num sum  up
	       contribution from each DP
	  */

		PROC MEANS DATA=&prefix.sscp_c1(RENAME=(_TYPE_=_TYPE_1) 
	                                                               ) NWAY NOPRINT;
	   CLASS  _TYPE_1	 _NAME_ /MISSING;
			VARS  &independent_vars. /*_WEIGHT_  _EVENT_*/;
	    OUTPUT 	 OUT=dplocal.&prefix.hess_grad(TYPE=sscp DROP=_TYPE_ _FREQ_ RENAME=(_TYPE_1=_TYPE_))
						   SUM=;
	  RUN;

	  /* Extract Hessian and gradient*/

	    DATA  &prefix.Hessian1 
		           DPLocal.&prefix.Gradient(KEEP=&independent_vars.)
				   ;
		LENGTH _NAME_  $ 32 &independent_vars. 8;
			 	SET DPLocal.&prefix.hess_grad;

		  ARRAY var{*} &independent_vars.;

		  /*Use the same case in variable names as originaly specified. 
		   This is useful for reporting.*/

			 DO __i__=1 TO DIM(var);
			   IF lowcase(_NAME_)=lowcase(VNAME(var{ __i__ })) THEN _NAME_ =VNAME(var{ __i__ });
			 END;
			 DROP __i__;

			IF           _TYPE_="HESSIAN"     THEN OUTPUT &prefix.Hessian1;
			ELSE IF _TYPE_="GRADIENT"  THEN OUTPUT DPLocal.&prefix.Gradient;

			DROP _TYPE_ ;
	   RUN;

	   		 				   /* Hessian matrix sorted in the order of VAR_NUMB_POS*/
	 PROC SQL;
		   CREATE TABLE  DPLocal.&prefix.Hessian(DROP=VAR_NUMB_POS) AS
	  		  SELECT * 							
                             ,INDEXW(LOWCASE(STRIP("&independent_vars.")),LOWCASE(STRIP(_NAME_)))  AS VAR_NUMB_POS
							FROM &prefix.Hessian1
				ORDER BY  CALCULATED VAR_NUMB_POS
					 ;
		 QUIT;
		

          %END;   /* End of do-group for &pos_dp_cd_in_strata_vars. > 0 */


	                    /* Transpose Gradient vector */

		  PROC TRANSPOSE DATA= DPLocal.&prefix.Gradient OUT=DPLocal.&prefix.Gradient_T(RENAME=(COL1=&dependent_vars.)) ;
	       VAR &independent_vars.;
          RUN;
		 QUIT;


/* Solve Newton-Raphson linear equation. The solution gives changes in regression  
   coefficients \beta relative previous iteration */

%solve_linear_equations_reg(tbl_name_left_in=DPLocal.&prefix.Hessian
                                            ,tbl_name_right_in=DPLocal.&prefix.Gradient_T
										    ,tbl_name_ginv_out=DPLocal.&prefix.Ginv_Hessian
                                            , tbl_name_out=&prefix.P_Est_Deltas);

	DATA &prefix.P_Est_varN_prev;
	  LENGTH  Var_Numb 4 Variable $ 32;
       SET dplocal.&prefix_prev.P_Est(KEEP=Variable DF Estimate) ;
	  Var_Numb=_N_;
	RUN;

	          /* Get new \beta by adding prev \beta and differece between new and prev \beta*/

		PROC SQL;
           CREATE TABLE &prefix.P_Est  AS 
		      SELECT    A.Var_Numb
                               ,A.Variable
							  ,DF
							  ,SUM(A.Estimate,B.Value) AS Estimate
							 ,A.Estimate						 AS Estimate_prev
			   FROM 			&prefix.P_Est_varN_prev   A
			   INNER JOIN  &prefix.P_Est_Deltas		      B
                    ON lowcase(A.Variable)= lowcase(B.Variable) 
					ORDER BY Var_Numb
					;
			QUIT;

			DATA dplocal.&prefix.P_Est ;
			 	SET &prefix.P_Est;
				  DROP Estimate_prev;
			RUN;
			
   	PROC TRANSPOSE DATA=dplocal.&prefix.P_Est 
			                           OUT=&prefix.Model_Coeff(DROP=_NAME_);
	  VAR  Estimate;
	   ID Variable;
  RUN;

	DATA DPLocal.&prefix.Model_Coeff (TYPE=PARMS);
	LENGTH  _Model_ $ 32 _TYPE_ $ 8;
	   SET 	&prefix.Model_Coeff;
			   _Model_="_XBETA_";
			 _TYPE_="PARMS"	;
			 
			 _IN_=&count_independent_vars.;
			 _P_=_IN_ ;	  /* No intercept for Cox */
			 _RMSE_=1;	/*Add this var to have the same structure as linear or logistic  Not really needed for Cox */
	RUN;

	/*%IF &last_iter_in. NE 1 %THEN */
	%IF &iter_nb. LE &iter_nb_at_conv. %THEN 
            %DO;

  	DATA infolder.&prefix.Model_Coeff ;
		 SET dplocal.&prefix.Model_Coeff;
   IF &regr_type_cd.=1 THEN _PHI_=_RMSE_**2; 
   ELSE 					               _PHI_=1;
  RUN;

 %check_track_convergence(tbl_p_est_in=dplocal.&prefix.P_Est
					                             ,tbl_prev_p_est_in=dplocal.&prefix_prev.P_Est
					                             ,tbl_p_est_tr_in=dplocal.&prefix.Model_Coeff
					                             ,tbl_convrg_status_out=&prefix.Convrg_Status
					                             ,tbl_Iter_parms_hist_updt=dplocal.&prefix0.Iter_Parms_hist
                                                  );

	%IF &reg_conv_in.=1	%THEN %LET reg_conv_msg=SUCCESS;

  %PUT reg_conv_in=&reg_conv_in.;
  %PUT reg_conv_msg=&reg_conv_msg.;
  %PUT iter_nb_at_conv=&iter_nb_at_conv.;

 /*	%IF &prev_reg_conv_in.=0 AND   &reg_conv_in.=1 %THEN  %LET reg_conv_step_in=1;	*/

	 %IF &iter_nb.=&iter_nb_at_conv.  %THEN  %LET reg_conv_step_in=1;

	  	%IF  &reg_conv_step_in. =1 %THEN
                    %DO;

		 DATA msoc.&prefix0.Model_Coeff ;
			 SET infolder.&prefix.Model_Coeff;
		RUN;

		DATA msoc.&prefix0.Convrg_Status;
	        LENGTH Status 8 Status_Dsc $ 20  Iteration 8 Reason $ 90;

            SET &prefix.Convrg_Status;

		 Iteration=iter_nb;
		 Status_Dsc="Not Converged";

		   IF reg_conv_in=1  THEN 
								  DO;
		 Status=0;
		 Status_Dsc="Converged";
        Reason="Convergence criterion (XCONV=&xconv.) satisfied."	 ;
		                          END;
		   ELSE IF iter_nb =&max_iter_nb. 
            THEN 
                    DO;
                Status=2;
Reason="Iteration limit reached without convergence. Max_iter_nb=&max_iter_nb.. (XCONV=&xconv.)" 	;
                    END;
		   ELSE DO; Status=3; Reason="Possible errors. Check logs and tracking table for details.";	END;

		  LABEL Status_Dsc="Convergence Status";
	  RUN;

		                   %END;	/* End of do-group for &reg_conv_step_in. =1 */

				    %END;	/* End of do-group for  &iter_nb. LE &iter_nb_at_conv.  */

	  %IF &iter_nb=1 %THEN
	                            %DO;
 	PROC MEANS DATA=&prefix0.total_cnts_c NWAY  NOPRINT;
			 VAR Total _Event_  _WEIGHT_ ;
	       OUTPUT 	 OUT=&prefix0.cens_sum( DROP=_TYPE_ _FREQ_ )
        					SUM=;
	    RUN;
 	  QUIT;

	    DATA  &prefix0.cens_sum;
		 LENGTH Stratum 8;
		   SET &prefix0.cens_sum;
			Stratum=1;
		 RUN;

	   %IF %LENGTH(&strata_vars.) >0 %THEN
	                     %DO;	
		 PROC MEANS DATA=&prefix0.total_cnts_c NWAY  NOPRINT;
	    CLASS &strata_vars.	;
			 VAR Total _Event_  _WEIGHT_ ;
	       OUTPUT 	 OUT=&prefix0.cens_sum_s( DROP=_TYPE_ _FREQ_ )
        					SUM=;
	    RUN;
 	  QUIT;

	    DATA  &prefix0.cens_sum_s
		           &dplocal_lib..&prefix0.Stratum_map(KEEP=Stratum &strata_vars.)
                  ;
		 LENGTH Stratum 8;
		  SET &prefix0.cens_sum_s ;
		  Stratum=_N_;
		  RUN;

		 DATA &dplocal_lib..&prefix0.mean_vars_by_st;
		    LENGTH Stratum 8;
		     MERGE &dplocal_lib..&prefix0.mean_vars_by_st(IN=in_mean)
			               &dplocal_lib..&prefix0.Stratum_map;
				BY	&strata_vars.	;
				 IF  in_mean;
		   RUN;

		 DATA  &prefix0.cens_sum;
		  SET &prefix0.cens_sum_s  &prefix0.cens_sum(IN=in_t);
		    IF in_t THEN Stratum=.;
		  RUN;
						 %END;

	   %ELSE
	    				 %DO;
			 DATA &dplocal_lib..&prefix0.mean_vars_by_st;
			   LENGTH Stratum 8;
				  SET &dplocal_lib..&prefix0.mean_vars_by_st;
					Stratum=1;
			RUN;
						 %END;

	     DATA msoc.&prefix0.cens_sum;
		 SET &prefix0.cens_sum(RENAME=( _Event_=Event _WEIGHT_=_SUMWGT_));
		   	 Censored=Total - Event;
			 PctCens=100*Censored/Total;
			 FORMAT PctCens 8.2;
		   KEEP Stratum &strata_vars. Total Event Censored PctCens _SUMWGT_;
	   RUN;

					  %END;	 /* End of do-group for &iter_nb=1*/

  %PUT;
   %PUT -----   End of the macro calc_cox_iter_stats_central  iter_nb=&iter_nb. ;
   %PUT;

%MEND;