
  %MACRO calc_final_stats_genmod_central;
		%LOCAL  _cl  modelfit_stats;
         /* Wrapper for calculating residuals at central siite. It can only be used in the context of distributed regression macro */

		  

 %IF &regr_type_cd. = 1 %THEN 
			           %DO;
 DATA  &prefix.P_Est_2;
   LENGTH  Var_Numb 4;
   SET dplocal.&prefix_prev.P_Est;
		   Var_Numb=_N_;
   RUN;

   	DATA msoc.&prefix0.Cov_Est ;
		 SET dplocal.&prefix_prev.Cov_Est;			
	RUN;

		DATA msoc.&prefix0.INVXPX;
		SET dplocal.&prefix_prev.INVXPX;
		  IF STRIP(LOWCASE(Variable)) NE STRIP(LOWCASE("&dependent_vars."));

		DROP  MODEL Dependent  &dependent_vars.;
	RUN;
					 %END;
	%ELSE
	                  %DO;

	   /* Update iteration history with Neg2LogLike information based on the final regression coefficients.
		      Not applicable to linear regression. 
	    */

		  DATA  &prefix.Neg2LogLike;
			  LENGTH iter_nb Neg2LogLike 8;
				SET DPLocal.&prefix.resid_sum(KEEP=_LL__Sum _L__Sum);
				iter_nb=&iter_nb.-1;
				Neg2LogLike=-2*_L__Sum;
		   RUN;

			DATA    dplocal.&prefix0.Iter_Parms_hist;
			 MERGE  dplocal.&prefix0.Iter_Parms_hist
                             &prefix.Neg2LogLike
						;
                BY iter_nb;
 		   RUN;
		
	   DATA msoc.&prefix0.Iter_Parms_hist;
	  	 SET dplocal.&prefix0.Iter_Parms_hist;
		  Iteration=iter_nb;
	  RUN;

		/*20160705. We will use parameter estimates from the iteration  at which
		 process converged (iter_nb=i) while Cov matrix and StdErr from the next iteration i+1. 
		 The difference is small but this is what SAS Logistic/Genmod uses and we adopt the same 
		 approach.
		*/

 DATA &prefix.P_Est_varN_prev;
	  LENGTH  Var_Numb 4;
       SET dplocal.&prefix_prev.P_Est(KEEP=Model Dependent Variable DF Estimate) ;
	  Var_Numb=_N_;
	   alpha=&alpha.;
	RUN;
	
 DATA &prefix.P_Est_varN;
	  LENGTH  Var_Numb 4;
       SET  dplocal.&prefix.P_Est(KEEP=StdErr) ;
	   	IF _N_=1 THEN	SET dplocal.&prefix.Model_Coeff(KEEP=_RMSE_ );
	       Var_Numb=_N_;

			/*  For non-linear regression the scale parameter is not necessarily equal to _RMSE_**2
		   calculated by linear regression at the covergence step. It is only true when SCALE=PEARSON
		   and  the scale parameter  is calculated using standard formula for Pearson ChiSq statistic 
			withoug any prior aggregation(option AGREGATE in logistic and GENMOD). However, for binary
			outcome the standard ChiSq statistic is inappropriate without AGGREGATE option. 
		   */
		 StdErr= StdErr/_RMSE_;
		 DROP _RMSE_ ;
	RUN;

	DATA  &prefix.P_Est_2;
	  MERGE &prefix.P_Est_varN_prev
	                  &prefix.P_Est_varN
                      ;
		BY 	Var_Numb;
	RUN;

	/* For Logistic the next two matrices should be the same because the scale parameter
	 _PHI_=1 for logistic.*/

	DATA msoc.&prefix0.Cov_Est ;
		 SET dplocal.&prefix.Cov_Est;
		   ARRAY indep{*} &Intercept. &independent_vars. ;
			IF _N_=1 THEN	SET dplocal.&prefix.Model_Coeff(KEEP=_RMSE_ );
				/*See comments for msoc.&prefix0.P_Est*/	 
	                       DO i=1 TO DIM(indep);
								   indep{i}=indep{i}/_RMSE_**2;							  
								END;
	 DROP i _RMSE_;
	RUN;

	DATA msoc.&prefix0.INVXPX;
		SET dplocal.&prefix.INVXPX;
		  IF STRIP(LOWCASE(Variable)) NE STRIP(LOWCASE("&dependent_vars."));

		DROP  MODEL Dependent  &dependent_vars.;
	RUN;
								%END;

 	DATA msoc.&prefix0.P_Est ;
	  LENGTH  Var_Numb 4;
	*  LENGTH Var_Numb 4 Model $ 32 Dependent $ 32 Variable $ 14 DF Estimate StdErr  tValue Probt 
					LowerCL  UpperCL 8  ;

	    SET &prefix.P_Est_2;
		 
			 alpha=&alpha.;

			IF &regr_type_cd. NE 1 THEN 
			           DO;		
             IF  StdErr>0 THEN 
								DO;				 
				 
				  tValue=Estimate/StdErr;

		/* For non-linear models the T-statistic is no better in small samples then Z-statistic 
		    and it use can be missleading. Even for linear models the former is true only when
		    error term have normal distribution which is almost never true with real data.
		   The Z-statistic can be justified for large samples using central limit theorem and does not 
 		   require normality of the error term.	For this reason we recalculate results PValue and
		  confidence limits. This makes them consistent with output from SAS Logistic/Genmod procedures.  
		*/
				   ProbT=2*(1-PROBNORM(abs(tValue))) ;
			      Quantile=PROBIT((1- alpha/2));
				 
				  LowerCL=Estimate-Quantile*StdErr;
				 UpperCL=Estimate+Quantile*StdErr;
								END;
					   END;	 /*End og &regr_type_cd. NE 1*/
      
		   DROP MODEL Quantile  ;
	 RUN;
 
   /*   The msoc.&prefix0.resid_sum dataset  has final overall sum of residuals, response, deviance, 
	    log-likelihood, AIC, etc.	
	*/

		DATA msoc.&prefix0.resid_sum;
			SET dplocal.&prefix.resid_sum;
		IF _N_=1 THEN	SET msoc.&prefix0.Model_Coeff(KEEP=_IN_ _P_  _PHI_ _RMSE_ _RSQ_ _ADJRSQ_);

	ARRAY R_Sum{*}  _RESP__Sum  _MEAN__Sum _RESIDUAL__Sum  _PRESIDUAL__Sum _DRESIDUAL__Sum;
	ARRAY R_Mean{*}  _RESP__Mean  _MEAN__Mean _RESIDUAL__Mean  _PRESIDUAL__Mean _DRESIDUAL__Mean;

			  DO i=1 TO DIM(R_Sum);
			       /* _RMSE_ which is unbiased estimate of \sigma is normolized by n-p rather than by n. */
				R_Mean{i}=R_Sum{i}/(Nobs);
			  END;	

			  _RESP_W__Mean=_RESP_W__Sum/_SUMWGT_; 
			  _MEAN_W__Mean=_MEAN_W__Sum/_SUMWGT_; 
         
      		  Scaled_Pearson_norm =_PRESIDUAL_SQ__Sum/(Nobs- _P_);
			  Scaled_Deviance_norm =_DRESIDUAL_SQ__Sum/(Nobs- _P_);

			   _AIC_=-2* _LL__Sum+2*_P_;
			  _AICC_=-2* _LL__Sum+2*_P_*(Nobs/(Nobs-_P_-1));
			  _BIC_=-2* _LL__Sum+_P_*LOG(Nobs);

	 /*To be consistent with GENMOD terminology multiply by _PHI_. 
	   We defined _DEV_ as scaled deviance(devide by _PHI_) */
			  _Deviance_=_DEV__Sum*_PHI_;

			  /*Define very small number. It is used to avoid numerical exception in Log and devision */
					_sml_=1E-12;

			  regr_type_cd=&regr_type_cd.;
			  _pi_=CONSTANT("pi"); 

			  /* Mean for intercept only model. */
            /*Add very small  number to avoid numeric exception in LOG */
				   _MEAN0_=_RESP_W__Mean;
				 _1_MEAN0_=1- _RESP_W__Mean;

				  intercept_in=_P_ -_IN_;
			 
			 /* Calculate generalized R-Square (Cox & Snell) and Global likelihood ratio test (comparisson with intercept only model) */
			  /* For linear regression _G_RSQ_ is almost the same as _RSQ_. They differ by very small factor 1/Nobs.
				We will use usual  _RSQ_ in this case to be consistent with PROC REG output. 
				For logistic _G_RSQ_ is consistent with output  for R-Square from PROC LOGISTIC */

				   IF regr_type_cd=1 THEN
				           DO;
				  /* Calculate stats for intercept only model */
				   _RMSE0_=_RMSE_/SQRT(1-_ADJRSQ_);
				     _L0_=-0.5*(Nobs-intercept_in +Nobs*log(2*_pi_ *_RMSE0_**2));
				 	
 				   _G_RSQ_=_RSQ_;
				   _G_RSQ__Max_Rescaled=_RSQ_;
							END;
						                                                  /* Logistic regression */
				ELSE IF regr_type_cd=2 THEN
				           DO;
 		/*Add very small  number to avoid numeric exception in LOG */
				   _MEAN0_=MAX(ABS(_MEAN0_),_sml_);
					_1_MEAN0_=MAX(ABS(_1_MEAN0_),_sml_);

				 _L0_=_SUMWGT_*(_MEAN0_*LOG(_MEAN0_) +( _1_MEAN0_)*LOG( _1_MEAN0_) )/ _PHI_ ;
 				    _G_RSQ_=1-exp(2*(_L0_ - _L__Sum)/Nobs);
				    _G_RSQ__Max_Rescaled=_G_RSQ_/(1-exp(2*(_L0_ )/Nobs));
						  END;
				  ELSE IF regr_type_cd=3 THEN
				           DO;
 				  _MEAN0_=MAX(ABS(_MEAN0_),_sml_);
				 _L0_=_SUMWGT_*(_MEAN0_*LOG(_MEAN0_) - _MEAN0_)/ _PHI_ ;
 					_G_RSQ_=1-exp(2*(_L0_ - _L__Sum)/Nobs);
				   _G_RSQ__Max_Rescaled=_G_RSQ_/(1-exp(2*(_L0_ )/Nobs));
						  END;

					_2LogLike_Ratio=2*(_L__Sum -_L0_);
					PValue_Glob_Null_Test=1-PROBCHI( _2LogLike_Ratio, _IN_);

		 	   DROP i _RESP__Sum  _MEAN__Sum _RESP_W__Sum _MEAN_W__Sum  
                        _RESIDUAL__Sum _DEV__Sum _PRESIDUAL__Sum _DRESIDUAL__Sum
                      _pi_ _sml_   _MEAN0_  _1_MEAN0_ intercept_in _RMSE0_;
      RUN;

	  
		   PROC DATASETS LIB=msoc NOLIST;
			 MODIFY &prefix0.resid_sum;
				 LABEL 
				  _Deviance_="Deviance"
				  Scaled_Deviance_norm="Deviance /DF"
				  _PRESIDUAL_SQ__Sum="Pearson Chi-Square"
				   Scaled_Pearson_norm="Pearson Chi-Square /DF "
				  _L__Sum="Log Likelihood"
				 _LL__Sum="Full Log Likelihood" 
				_AIC_ ="AIC (smaller is better)"
                _AICC_ ="AICC (smaller is better)"
                _BIC_="BIC (smaller is better)"
				_G_RSQ_ ="R-Square "
                _G_RSQ__Max_Rescaled="Max-rescaled R-Square"   
			 ;
		  RUN;

		  /* Do not include _Deviance_   _PRESIDUAL_SQ__Sum for logistic because they 
		      do not have ChiSq distribution are not very useful in this case */

%LET modelfit_stats=  %SYSFUNC(COMPBL(_Deviance_   _PRESIDUAL_SQ__Sum   _L__Sum 
                                        _LL__Sum  _AIC_ _AICC_  _BIC_ 
                                        _G_RSQ_  _G_RSQ__Max_Rescaled));
%IF &regr_type_cd. = 2  
  %THEN %LET modelfit_stats=%SYSFUNC(COMPBL(_L__Sum _LL__Sum  _AIC_ _AICC_  _BIC_ 
                                                                                            _G_RSQ_  _G_RSQ__Max_Rescaled));  ;


		  DATA &prefix0.modelfit_tr;
		  LENGTH _Deviance_  Scaled_Deviance_norm _PRESIDUAL_SQ__Sum  Scaled_Pearson_norm 
                           _L__Sum _LL__Sum  _AIC_ _AICC_  _BIC_ _G_RSQ_  _G_RSQ__Max_Rescaled 8 ;
			 SET 	msoc.&prefix0.resid_sum;

			 KEEP &modelfit_stats.;
		 RUN;

		 PROC TRANSPOSE DATA=&prefix0.modelfit_tr 
                                        OUT=&prefix0.modelfit1(RENAME=(COL1=Value));

        RUN;

		DATA msoc.&prefix0.modelfit;
		 LENGTH _NAME_  Criterion $ 32  DF Value ValueDF 8;
		  SET &prefix0.modelfit1;
		   IF _N_=1 THEN SET msoc.&prefix0.resid_sum(KEEP=Nobs _P_);
		                         Criterion=_LABEL_;
								  
                 IF UPCASE(_NAME_) IN ("_DEVIANCE_"	"_PRESIDUAL_SQ__SUM") THEN 
		     							DO;
						 DF=Nobs - _P_	;
						  ValueDF=Value/DF;
										END;
			  FORMAT  DF 12.  Value ValueDF	 12.4;
			KEEP _NAME_ Criterion DF Value ValueDF;
		  RUN;    


		  DATA msoc.&prefix0.Glob_Null_ChiSq	;
		   LENGTH Test $ 16 ChiSq DF ProbChiSq  8;
		    SET 	msoc.&prefix0.resid_sum(KEEP=_2LogLike_Ratio _IN_ PValue_Glob_Null_Test 
															     RENAME=(_2LogLike_Ratio= ChiSq _IN_=DF PValue_Glob_Null_Test=ProbChiSq)
	                                                            );
			Test="Likelihood Ratio";
			 FORMAT ProbChiSq  PVALUE6.4;
			    LABEL 	Test="Test"
	                          ChiSq="Chi-Square"	 /* Pr > ChiSq */
							  DF="DF"
							  ProbChiSq="Pr > Chi-Square"
				              ;
		   RUN;

		  /* The steps below calculate robust sandwich estimate of 
		     cov matrix (heteroscedasticity-consistent HC cov matrix)
		   */ 

	  /*  Summarize contributions from all DPs*/

	 	   PROC MEANS DATA=&prefix0.sw_matrix_c(RENAME=(_TYPE_=_TYPE_1) 
																	WHERE=(_TYPE_1   IN ("SSCP" ) )
	                                                               ) NWAY NOPRINT;
	   CLASS _TYPE_1	 _NAME_/MISSING;
	    OUTPUT 	 OUT=&prefix0.sw_matrix_s( DROP=_TYPE_ _FREQ_ _TYPE_1)
						   SUM=;
	  RUN;

	  
		 PROC SQL;
		   CREATE TABLE  &prefix0.sw_matrix_s1(DROP=VAR_NUMB_POS) AS
	  		  SELECT * 							
                             ,INDEXW(LOWCASE(STRIP("Intercept &independent_vars.")),LOWCASE(STRIP(_NAME_)))  AS VAR_NUMB_POS
							FROM &prefix0.sw_matrix_s
				ORDER BY  CALCULATED VAR_NUMB_POS
					 ;
		 QUIT;

		  DATA 	&prefix0.sw_matrix_s2;
			 SET &prefix0.sw_matrix_s1;
			  ARRAY indep{*} &Intercept. &independent_vars. ;
		        IF _N_=1 THEN	SET msoc.&prefix0.resid_sum(KEEP =Nobs _P_ );

			  /* The factor Nobs/(Nobs - _P_) correspond to HC1 definition HC cov matrix in PROC REG */

				 DO i=1 TO DIM(indep);
				   indep{i}=indep{i}* (Nobs/(Nobs - _P_)) ;
				 END;
				 DROP i Nobs _P_ ;
		   RUN;

	   %calc_matrix_product(tbl_name_left_in=&prefix0.sw_matrix_s2
                                             ,tbl_name_right_in =msoc.&prefix0.INVXPX
                                             ,tbl_name_out=&prefix0.I1PinvXPX
						                     ,tbl_left_col_with_row_nm=_NAME_
											 ,tbl_right_col_with_row_nm=Variable
                                            ,tbl_right_is_symmetric_flag=1);

		    %calc_matrix_product(tbl_name_left_in=msoc.&prefix0.INVXPX
                                             ,tbl_name_right_in =&prefix0.I1PinvXPX
                                             ,tbl_name_out=msoc.&prefix0.HC_Cov
						                     ,tbl_left_col_with_row_nm=Variable
											 ,tbl_right_col_with_row_nm=_NAME_
                                            ,tbl_right_is_symmetric_flag=0);


	  /* The dataset msoc.&prefix0.HC_Cov has heteroscedasticity-consistent cov matrix*/

            DATA  msoc.&prefix0.HC_Cov;
			   LENGTH Model Dependent $ 32;
			     SET msoc.&prefix0.HC_Cov;
				     Model="_XBETA_";
					Dependent="&dependent_vars." ;
			  RUN;

		   DATA &prefix0.HC_StdErr;
			 LENGTH   Var_Numb 4;
		      SET msoc.&prefix0.HC_Cov;
				ARRAY indep{*} &Intercept. &independent_vars. ;
	
					 DO Var_Numb=1 TO DIM(indep);			
				      IF STRIP(LOWCASE(Variable))=STRIP(LOWCASE( VNAME(indep{Var_Numb}) ))	
					   THEN DO;
						  /*Negative values are not expected. This just defensive coding.*/
				IF 	indep{Var_Numb}>=0	THEN	HCStdErr=SQRT(indep{Var_Numb});
				ELSE 											 HCStdErr=.;
					  HCCMethod="HC1";
     							OUTPUT;
							   END;
				 END; 
				 FORMAT HCStdErr D11.3;
				KEEP Variable HCStdErr HCCMethod Var_Numb;
		  RUN;

			DATA   msoc.&prefix0.P_Est_HC(DROP=Var_Numb) ;
			  LENGTH Var_Numb 4;
			   MERGE msoc.&prefix0.P_Est
							&prefix0.HC_StdErr
						  ;				
				 BY Var_Numb;

		FORMAT HCTValue 7.2 HCProbt PValue6.4 HCLowerCL HCUpperCL D11.3;

	IF _N_=1 THEN	SET msoc.&prefix0.resid_sum(KEEP =Nobs _P_ );

   IF  HCStdErr>0 THEN DO;
				  df_n_p=Nobs -_P_;
				  HCTValue=Estimate/HCStdErr;

	IF &regr_type_cd.= 1 THEN 
			           DO;
				  HCProbt=2*(1-probt(abs(HCTValue),df_n_p)) ;
				  Quantile=TINV((1-alpha/2),df_n_p);
					   END;
	ELSE
	 				DO;
			HCProbT=2*(1-PROBNORM(abs(HCTValue))) ;
			Quantile=PROBIT((1-alpha/2));
				  END;

				  HCLowerCL=Estimate-Quantile*HCStdErr;
				 HCUpperCL=Estimate+Quantile*HCStdErr;
									  END;
				*  alpha=&alpha.;

				 DROP Quantile; 
			  RUN;

		DATA  _NULL_;
		   CALL SYMPUT("_cl",STRIP(PUT(100*(1.0-&alpha.),best12.)));
		RUN;
			 
			   %PUT _cl=&_cl.;

		   PROC DATASETS LIB=msoc NOLIST;
			 MODIFY &prefix0.P_Est_HC;
				 LABEL 
				  Estimate="Parameter Estimate"                                   
                  StdErr="Standard Error"                                         
                   tValue="t Value"                                                
                   Probt="P-Value" 
                    
                   HCStdErr="Heteroscedasticity Consistent Standard Error"         
                   HCCMethod="Heteroscedasticity Consistent Covariance Method"     
                   HCTValue="Heteroscedasticity Consistent t Value"                
                   HCProbt="Heteroscedasticity Consistent P-Value"                
                   HCLowerCL="Heteroscedasticity Consistent Lower &_cl.% CL Parameter"
                   HCUpperCL="Heteroscedasticity Consistent Upper &_cl.% CL Parameter"
                   LowerCL="Lower &_cl.% CL Parameter"                                
                   UpperCL="Upper &_cl.% CL Parameter"  		
			 ;
		  RUN;

 /* The  dataset msoc.&prefix0.resid_sum_by_pct and  msoc.&prefix0.resid_sum_by_pct2
	 has data groupd data by percentiles of predicted outcome \mu. The number of groups (bins)
	for msoc.&prefix0.resid_sum_by_pct is determined by parameter &groups and a number 
	groups for msoc.&prefix0.resid_sum_by_pct2 is determined by the parameter &min_count_per_grp. 
*/
			  DATA msoc.&prefix0.resid_sum_by_pct;
	                SET &prefix0.resid_sum_by_pct_c;
			  RUN;

			    DATA msoc.&prefix0.resid_sum_by_pct2;
	                SET &prefix0.resid_sum_by_pct2_c;
			  RUN;

			  

	   %IF &regr_type_cd.  =2  %THEN 
			           %DO;

  /* Calculate HosmerLemeshow statistic*/

%calc_hl_stats_central(tbl_name1_in=msoc.&prefix0.resid_sum_by_pct
									   ,tbl_name2_in=msoc.&prefix0.resid_sum_by_pct2
                                       ,tbl_name_hl_partition_out=msoc.&prefix0.hl_partition
                                       ,tbl_name_hl_chisq_out=msoc.&prefix0.HL_ChiSq
									   ,groups=&groups.
                                       ,prefix0=&prefix0.);

	
							  /* Calculate ROC*/

 DATA &prefix0.data_for_roc;
	  LENGTH  _PROB_ 8;
			   SET msoc.&prefix0.resid_sum_by_pct2(KEEP= _FREQ_ _MEAN__Mean  _RESP__Mean )
                           END=last;

			   _PROB_=_MEAN__Mean;
			  _RESP_=_RESP__Mean*_FREQ_;
						;
	   DROP _MEAN__Mean _RESP__Mean;
			 RUN;

 /* Create dataset for ROC curve and binary response (logistic regression) */

%calc_roc( tbl_name_in=&prefix0.data_for_roc
                   ,tbl_name_out=msoc.&prefix0.ROC);

						%END;

%PUT  ------  End of macro calc_residual_stats_central.;
%MEND; 	