  %MACRO calc_genmod_iter_stats_central;

	  /* Wrapper for calculating various Cox regression stats per iteration at the central site. */

   %PUT;
   %PUT -----   Start of the macro calc_genmod_iter_stats_central  iter_nb=&iter_nb. ;
   %PUT;

     /*  Summarize contributions from all DPs for overall sum of residuals, response, deviance, etc.
         This is done for each iteration except for linear regr. in which case it is only done &iter_nb.=2	   */

	 %IF  &regr_type_cd.  NE 1 OR  &iter_nb. > 1  %THEN
 				%DO;
  PROC MEANS DATA=&prefix.resid_sum_c NWAY NOPRINT;
			 VAR _FREQ_  _SUMWGT_ 
                     _RESP__Sum  _MEAN__Sum 
                     _RESP_W__Sum  _MEAN_W__Sum
                     _L__Sum  _LL__Sum 
                    _DEV__Sum _RESIDUAL__Sum   
                    _PRESIDUAL__Sum _PRESIDUAL_SQ__Sum 
					 _DRESIDUAL__Sum  _DRESIDUAL_SQ__Sum
						;
	       OUTPUT 	 OUT=DPLocal.&prefix.resid_sum( DROP=_TYPE_ _FREQ_ )
						    SUM(_FREQ_)=Nobs  
							MIN(Min__MEAN_)=Min__MEAN_ MAX(Max__MEAN_)=Max__MEAN_ 
        					SUM=;
	    RUN;

				   %END;

				   /*20160705. Calculate SSCP for last &last_iter_in as well. It will be used to calculate
		 covariance matix of estimates but not the estimates itself. The later come from the iteration
		 at which convergence was achived.*/

			 /* Only records with TYPE="SSCP" will be used in the PROC REG. 
				 We excllude TYPE  IN ("MEAN", "STD") because summing them is not meaningful  */

   %IF %LENGTH(&_sscp_input_ds_sh.) >0 %THEN
									 %DO;
		
	   PROC DATASETS  MEMTYPE=DATA NOLIST;
		   MODIFY &prefix.sscp_c;
		     ATTRIB _ALL_ LABEL=" ";
		     ATTRIB _ALL_ FORMAT=;
		RUN;
		QUIT;

		 /* Defensive coding in case data partners use different case for vars in their input dataset.
			Although SAS is not case sensitive as far as variable names are concerned this affects 
			the case of the values in the _NAME_ variable of the sscp dataset.
		 */
		
	 DATA &prefix.sscp_c;
		SET &prefix.sscp_c;
		   ARRAY var{*} &independent_vars. &dependent_vars. &strata_vars.;

			 DO i=1 TO DIM(var);
			   IF lowcase(_NAME_)=lowcase(VNAME(var{i})) THEN _NAME_ =VNAME(var{i});
			 END;
			 DROP i;
	 RUN;

	    /*  Summarize contributions from all DPs	   */

	  PROC MEANS DATA=&prefix.sscp_c(RENAME=(_TYPE_=_TYPE_1) 
																	WHERE=(_TYPE_1  NOT IN ("MEAN", "STD") )
	                                                               ) NWAY NOPRINT;
	   CLASS _TYPE_1	 _NAME_/MISSING;
	    OUTPUT 	 OUT=DPLocal.&prefix.SSCP(TYPE=SSCP DROP=_TYPE_ _FREQ_ RENAME=(_TYPE_1=_TYPE_))
						   SUM=;
	  RUN;

	   /* Calculate regression coefficients, cov matrix, etc., using SSCP dataset as input
		  Output datasets are stored in DPLocal.
	  */

	%lin_reg_with_sscp_input(tbl_name_in=DPLocal.&prefix.SSCP 
													   ,dependent_vars=&dependent_vars.
													   ,independent_vars=&independent_vars.
													   ,NOINT=&NOINT.
													   ,model_label=_XBETA_
													   ,prefix=&prefix.
													   ,lib_out=DPLocal
														) ;
									%END;

 %IF &last_iter_in. NE 1 %THEN 
                                                             %DO;

   /*_PHI_ is a scale parameter in GENMOD */

   DATA infolder.&prefix.Model_Coeff ;
		 SET dplocal.&prefix.Model_Coeff;
   IF &regr_type_cd.=1 THEN _PHI_=_RMSE_**2; 
   ELSE 					               _PHI_=1;
  RUN;

	 	 /* For linear convergence is set to 1 after the first  itteration (iter_nb=1 and step_nb=2) provided
		   there were no preceding errors. 
		*/

    %IF  &regr_type_cd. NE 1 %THEN
								%DO;
	         
 %check_track_convergence(tbl_p_est_in=dplocal.&prefix.P_Est
					                             ,tbl_prev_p_est_in=dplocal.&prefix_prev.P_Est
					                             ,tbl_p_est_tr_in=dplocal.&prefix.Model_Coeff
					                             ,tbl_convrg_status_out=&prefix.Convrg_Status
					                             ,tbl_Iter_parms_hist_updt=dplocal.&prefix0.Iter_Parms_hist
                                                  );
			  
							   %END;  /* End of &regr_type_cd. NE 1 */

	%ELSE %IF &iter_nb. GE 1 AND &step_nb.=2 AND &regr_type_cd.=1 %THEN 
							   %DO;
		%LET reg_conv_in=1;
		%LET iter_nb_at_conv=1;
							  %END;

   %IF &reg_conv_in.=1	%THEN %LET reg_conv_msg=SUCCESS;

  %PUT reg_conv_in=&reg_conv_in.;
  %PUT reg_conv_msg=&reg_conv_msg.;

 	/*	%IF &prev_reg_conv_in.=0 AND   &reg_conv_in.=1 %THEN  %LET reg_conv_step_in=1;	*/

	 %IF &iter_nb.=&iter_nb_at_conv.  %THEN  %LET reg_conv_step_in=1;
	
	   /* Create a number of final output datasets once convergence is achieved
		  Some additional datasets (e.g. cov matrix for logistic) will requiere one more iteration because
         the SSCP dataset hast to be recalculated using final \beta.  They will be calculated
	     using the macro %calc_final_stats_genmod_central
	 */

	%IF  &reg_conv_step_in. =1 %THEN
                    %DO;
	 DATA msoc.&prefix0.Model_Coeff ;
		 SET infolder.&prefix.Model_Coeff;
	RUN;

	DATA msoc.&prefix0.FitStatistics ;
		 SET dplocal.&prefix.FitStatistics;
	RUN;

	DATA msoc.&prefix0.ANOVA ;
		 SET dplocal.&prefix.ANOVA;
	RUN;
	
	DATA msoc.&prefix0.CollinDiag ;
		 SET dplocal.&prefix.CollinDiag;
	RUN;

  %IF  &regr_type_cd. NE 1 %THEN
								%DO;
/*
	DATA infolder.&prefix0.Overall_stats ;
		 SET dplocal.&prefix.resid_sum;
  KEEP Nobs Min__MEAN_ Max__MEAN_ _RESP__Sum;
  RUN;
 */

	DATA msoc.&prefix0.Convrg_Status;
	LENGTH Status 8 Status_Dsc $ 20  Iteration 8 Reason $ 90;
	  	 SET &prefix.Convrg_Status;

		 Iteration=iter_nb;
		 Status_Dsc="Not Converged";

		   IF reg_conv_in=1  THEN 
								  DO;
		 Status=0;
		 Status_Dsc="Converged";
        Reason="Convergence criterion (XCONV=&xconv.) satisfied."	 ;
		                          END;
		   ELSE IF iter_nb =&max_iter_nb. 
            THEN 
                    DO;
                Status=2;
Reason="Iteration limit reached without convergence. Max_iter_nb=&max_iter_nb.. (XCONV=&xconv.)" 	;
                    END;
		   ELSE DO; Status=3; Reason="Possible errors. Check logs and tracking table for details.";	END;

		    PUT 	Status= Status_Dsc=  ;
	       PUT Reason=;

		  LABEL Status_Dsc="Convergence Status";
	  RUN;
								  %END;

				%END;  /*End of &reg_conv_step_in. =1 */

													 %END; /* &last_iter_in. NE 1*/ 
  %PUT;
   %PUT -----   End of the macro calc_genmod_iter_stats_central  iter_nb=&iter_nb. ;
   %PUT;

%MEND;