	 /* Wrapper for the first iteration step of DR regression at the central site
        First version:  04/07/2016, Yury Vilk  
		Modified 20170130 (modification for job_done.ok/ job_fail.ok). Y.V.
   */

%MACRO d_reg_central_step1;
	
	%LOCAL prefix2 step_return_cd step_return_msg tbl_list_not_transf addn_tbl_to_transf  tbl_list_to_transf;
	
	   /* Reset SAS automatic error variable. 
   Variable runid_return_cd keeps error cd from previous step if necessary */
	    %LET SYSCC=0; 
	
  %PUT;
   %PUT -----   Start of the macro d_reg_central_step1  iter_nb=&iter_nb. ;
   %PUT;


  /* In case of error the program will execute for the next few step before shutting down. 
	It will try to updated SAS table vars_nm_value_pairs with error info . 
  This will allow programs at DP's shutdown gracefully*/

/*	Commented staff below is for debugging only.

 %LET iter_nb=1;
%LET prefix0=&RunID._;
%LET prefix=&prefix0.i&iter_nb._;
%LET prefix2=&prefix.s&step_nb._;
%LET prefix_prev=&prefix0.i%eval(&iter_nb.-1)_  ;
*/

   /*
%LET iter_nb=2;
%LET prefix=&prefix0.i&iter_nb._;
%LET prefix2=&prefix.s&step_nb._;
%LET prefix_prev=&prefix0.i%eval(&iter_nb.-1)_  ;
 *%LET last_iter_in=1;
 */

  /*  %LET msoc_lib=msoc; */

	/* Initial cleanup to make sure that files from previous run/iteration will not be used. &RunID level*/
	
    %LET step_nb=1;
    %LET step_return_cd=0;
   %LET step_return_msg=PASS;

	 %PUT;
	 %PUT Start of step: iter_nb=&iter_nb. step_nb=&step_nb. ;
	 %PUT;

 %IF &iter_nb.=1 %THEN
  										%DO;  
	  %delete_ext_file(file_name=&infolder.job_done.ok, warning_flag=0);
       %delete_ext_file(file_name=&infolder.job_fail.ok, warning_flag=0);
	   
	    %delete_sas_tables(tbl_name_sh_list=&prefix0.:);
		 %delete_sas_tables(lib=infolder,tbl_name_sh_list=&prefix0.:);
	     %delete_sas_tables(lib=msoc,tbl_name_sh_list=&prefix0.:);
	     %delete_sas_tables(lib=dplocal,tbl_name_sh_list=&prefix0.:);

		 %cleanup_dp_msoc(dp_cd_list=&dp_cd_list.
                                        , tbl_name_sh_list=&prefix0.: 
                                        , file_nm_lst=files_done.ok job_done.ok job_fail.ok);

									   %END;

      /* Cleanup all files and SAS tables with names that will be created in the current iteration. */

 %delete_ext_file(file_name=&infolder.files_done.ok, warning_flag=0);
  %delete_ext_file(file_name=&infolder.file_list.csv, warning_flag=0);
  %delete_ext_file(file_name=&infolder.track_tbl.csv, warning_flag=0);

   %delete_sas_tables(lib=infolder, tbl_name_sh_list=vars_nm_value_pairs);

   %LET prefix2=&prefix.s&step_nb._;

  	%IF &runid_return_cd. > 0 %THEN
	             %DO;
   %LET step_return_cd=11;
 %LET  step_return_msg=Error in the previous step. Current step iter_nb: &iter_nb. step_nb: &step_nb..Run with RunID: &RunID. will terminate. ;
				 %END;

	%IF     &iter_nb.=%eval(&iter_nb_at_conv. + &numb_iter_after_conv.) 
         OR &iter_nb. >&max_iter_nb. 
         OR &runid_return_cd. > 0 
   %THEN 
					   %DO;
%LET last_iter_in=1;
%IF &last_runid_in.=1 OR &runid_return_cd. > 0 %THEN  %LET end_job_dp_in=1;
						%END;

%PUT prefix2=&prefix2.;
%PUT reg_conv_in=&reg_conv_in.;
%PUT last_iter_in=&last_iter_in.;
%PUT end_job_dp_in=&end_job_dp_in.;
%PUT runid_return_cd=&runid_return_cd.;

			      /* Initialize tracking table for a given itteration &iter and first step*/

  %initialize_tracking_tbl(dp_cd=0,tbl_name_out=&prefix2.tr_tb_updt1);
 	/* Create table with initial parameter estimates when necessary*/

	  %IF &regr_type_cd. NE 1 AND &iter_nb. = 1	%THEN
	            %DO;
		 %isdata(dataset=infolder.&tbl_intial_est.);
	   %IF &nobs.=0 %THEN
	                        %DO;			  
						DATA infolder.&prefix_prev.Model_Coeff;
						  LENGTH _MODEL_ $ 32  _TYPE_ $ 8;
						   ARRAY indep{*} &Intercept. &independent_vars. ;

						_MODEL_="_XBETA_";
						    _TYPE_="PARMS";
							_PHI_=1;

                       DO i=1 TO DIM(indep);
							   indep{i}=0;
							   DROP i;
							END;
						RUN;				
							%END;
		 %ELSE  %IF %lowcase(&tbl_intial_est.) NE  %lowcase(&RunID._i0_Model_Coeff) 
		   %THEN
		                   %DO;
						   	DATA infolder.&prefix_prev.Model_Coeff;
						 		SET infolder.&tbl_intial_est.;
								LENGTH 	_PHI_ 8;
								IF 	_PHI_ = . THEN 	_PHI_=1;
							RUN;
							%END;
			%ELSE ; /* Do nothing */

   	PROC TRANSPOSE DATA= infolder.&prefix_prev.Model_Coeff(KEEP=&Intercept. &independent_vars.)
			                                   OUT=DPLocal.&prefix_prev.P_EST(RENAME=(_NAME_=Variable COL1=Estimate))
									;
  RUN;

   DATA DPLocal.&prefix_prev.P_EST;
	  LENGTH Variable $ 32 DF 4;
	   SET DPLocal.&prefix_prev.P_EST;
				DF=1;
   RUN;

		   DATA dplocal.&prefix0.Iter_Parms_hist;
				LENGTH  iter_nb 8 LogLikelihood Neg2LogLike 8;
				 SET  infolder.&prefix_prev.Model_Coeff(KEEP=&Intercept. &independent_vars.);
				  iter_nb=0;
				   reg_conv_in=0;
                 LogLikelihood=.;
                 Neg2LogLike=.;	
			RUN;
						
					%END;  /* End of &regr_type_cd. NE 1 AND &iter_nb. = 1	 */	

  %IF &step_return_cd.=0 AND  &SYSCC. > 4  %THEN	 
 				 %DO;
 %LET  step_return_cd=100;
 %LET  step_return_msg=%STR(ERROR: Step fail iter_nb=&iter_nb. step_nb=&step_nb.. See log for details.);
  %PUT;
 %PUT  step_return_msg=&step_return_msg.;
 %PUT;
				%END;

%IF &step_return_cd. > 0	%THEN	%LET runid_return_cd=1;

%LET m_vars_list=%SYSFUNC(COMPBL(runid prefix0 reg_ds_in independent_vars Intercept dependent_vars freq weight noint  groups
								                        regr_type_cd  iter_nb prefix prefix_prev reg_conv_in last_iter_in  iter_nb_at_conv
                                                        numb_iter_after_conv last_runid_in end_job_dp_in runid_return_cd 
                                                         test_env_cd max_numb_of_grp min_count_per_grp_glob
                                                         &addn_cox_vars_for_dp. ));

   %create_parameters_tbl(m_vars_list=&m_vars_list.
										   ,tbl_name_out=DPLocal.&prefix.vars_nm_value_pairs
										  ,tbl_name_out2=infolder.vars_nm_value_pairs
                                           );
	
   %LET tbl_list_to_transf=vars_nm_value_pairs.sas7bdat  ; 
   %LET tbl_list_not_transf=&prefix0.track_tbl.csv  file_list.csv ; /* vars_nm_value_pairs_for_client.csv &prefix0.track_tbl_fl_lev.csv*/
   
	%IF &step_return_cd. >0 %THEN
                  %DO;	 				
	     %GOTO endmac;  
				  %END;

  %LET addn_tbl_to_transf=;

 	 /* For Cox regression */
	%IF &regr_type_cd. =10   %THEN
				        %DO;

		   %IF       (&exist_init_events_time_set. =1 AND &iter_nb.= 1)
	 			  OR (&exist_init_events_time_set. =0 AND &iter_nb.= 2 AND  &pos_dp_cd_in_strata_vars. =0)
			%THEN 	%LET addn_tbl_to_transf=&addn_tbl_to_transf.  &tbl_events_time_set..sas7bdat;

		/* Note that the condinition &iter_nb. = %eval(&iter_nb_at_conv. + 2) can be only satisfied
         if &pos_dp_cd_in_strata_vars. =0 and thus &numb_iter_after_conv=2 */

		%IF	&iter_nb. <= %eval(&iter_nb_at_conv. + 1) 
	     %THEN 	 %LET addn_tbl_to_transf=&addn_tbl_to_transf. &prefix_prev.model_coeff.sas7bdat;
		 %ELSE 	  %LET addn_tbl_to_transf=&addn_tbl_to_transf. &prefix0.baseln_hazard.sas7bdat;
		
						 %END;  /* End of do-group for &regr_type_cd. =10*/
 
						 /* Generalized Linear Model regression */
	   %ELSE %IF &regr_type_cd. NE 1 OR &iter_nb. > 1
	        %THEN 	%LET addn_tbl_to_transf=&addn_tbl_to_transf. &prefix_prev.model_coeff.sas7bdat;

/* If there is error then only sent parameter file which will trigger shutting down of all DP's processes*/
		%IF &runid_return_cd. NE 0 %THEN  %LET addn_tbl_to_transf=;	

   	  %LET tbl_list_to_transf=%SYSFUNC(COMPBL(vars_nm_value_pairs.sas7bdat  &addn_tbl_to_transf. ));

		%PUT tbl_list_to_transf=&tbl_list_to_transf.;
  
   %endmac:

%output_file_list(tbl_list_to_transf=&tbl_list_to_transf.
						    ,tbl_list_not_transf=&tbl_list_not_transf.
						    ,file_list_txt_out=&infolder.file_list.csv
						    ,file_list_tbl_out=&prefix2.file_list
							,file_list_tbl_out_for_csv=dplocal.&prefix2.file_list_for_csv
							,dir_nm=&infolder.);

  %find_files_mod_dtm( tbl_name_in=&prefix2.file_list , tbl_name_out=dplocal.&prefix2.file_list_dtm);


%update_tracking_tbl(updt_tbl_name_in=&prefix2.tr_tb_updt1
									  ,updt_tbl_name_out=dplocal.&prefix2.tr_tb_updt
									   ,tracking_tbl_out=infolder.&prefix0.track_tbl
                                      , file_list_txt_out=&infolder.&prefix0.track_tbl.csv
                                       );

 							/* Initialize tracking table on file level */
  %create_tracking_tbl_fl_lev_updt(	updt_tbl_name_in=dplocal.&prefix2.tr_tb_updt
										                     ,file_list_dtm_in=dplocal.&prefix2.file_list_dtm
															,updt_tbl_name_fl_lev_out=dplocal.&prefix2.tr_tb_fl_updt
															);

%update_tracking_tbl_fl_lev(updt_tbl_name_fl_lev_in=dplocal.&prefix2.tr_tb_fl_updt
                                                  ,tracking_tbl_fl_lev_out=infolder.&prefix0.track_tbl_fl_lev);


 %create_empty_file(file_nm=&infolder.files_done.ok);

  %IF &step_return_cd.=0 AND  &SYSCC. > 4  %THEN	 
 				 %DO;
 %LET  step_return_cd=100;
 %LET  step_return_msg=%STR(ERROR: Step fail iter_nb=&iter_nb. step_nb=&step_nb.. See log for details.);
 %PUT;
 %PUT  step_return_msg=&step_return_msg.;
 %PUT;
				%END;

%IF &step_return_cd. >0 %THEN
                  %DO;
%IF &runid_return_cd.=0 %THEN %DO;
		  %LET runid_return_cd=&step_return_cd.;
		 %LET runid_return_msg=&step_return_msg.;
		                								 %END;
				  %END;

     %PUT;
	 %PUT End of step: iter_nb=&iter_nb. step_nb=&step_nb. ;
	%PUT;
	 %PUT step_return_cd=&step_return_cd. ; 
     %PUT step_return_msg=&step_return_msg. ;
 	 %PUT;
     %PUT -----   End of the macro d_reg_central_step1;
    %PUT;
	%MEND;