    /***************************************************************************************************
*                                         
****************************************************************************************************
* NAME:  run_d_reg_central_templ
*
* PURPOSE: template for data partner master program for distributed regression;
*
*--------------------------------------------------------------------------------------------------
* HISTORY:
*  Create date (mm/dd/yy): 08/20/16,  Yury Vilk.
*  Last modified date (mm/dd/yy): 02/23/18,  Yury Vilk.
*
**************************************************************************************************/

/*-------------------------------------------------------------------------------------------------
*                                        DEFINITIONS
*--------------------------------------------------------------------------------------------------
*
* MSReqID  ... Mini-Sentinel Project Request-Id
*              NOTE: This parameter needs to be set by the programmer preparing the Request-ID
*                    in coordination with the Mini-Sentinel Operations Center (MSOC)
*
* The following paths for the current request are automatically derived 
*    DPLOCAL  ... path for data created that remains local (i.e behind the DP firewall)
*    MSOC     ... path for data created to share with MS Operations Center
*    INFOLDER ... path for program inputs (e.g.lookup data, external macros)
*    SASPROGRAMS ... path for all SAS programs that are part of the data request
*    MACROS			   ... path for all SAS macros that are part of the data request
*-------------------------------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------*/
/* NOTE: Analytic center site programmer defines parameters in this section */
/*----------------------------------------------------------------------------*/


/* The parameter dp_cd (data partner code) for the central site should be always set to 0. Do not edit*/
 %LET dp_cd=0;

   /* Specify list of data partner codes participating in distributed regression*/
         %LET dp_cd_list=1 2 3;

 * Edit root directory for all requests related for distrubuted regression at the central site. ;

 * %LET sites_prg_root_dp=\distrib_regr_all_requests_central\;
 %LET sites_prg_root_dp=J:\prshare\MSCDM\PRG\Mod_Prg_Physical_MSCDM\_Dev\Distributed_Regression\ver1\\dp_sites\dp0\;
* %LET sites_prg_root_dp=%STR(J:\FDA_Sentinel_ASPE_2014\Internal testing\dp_sites\dp0\);

/* 
  The MSReqID is defined at the central site before distributing request to all data partners.
   Should be the same as name of the subdirectory for the request.
  All data partners participating in the given request should have the same MSReqID.; 
*/
  
       *For testing use any name without spaces or special characters.  ;

  * %LET MSReqID = request_1;
* %LET MSReqID =v_glc_01;
 %LET MSReqID =v_glc_i01_pkg; 

    /* For production Sentinel requests follow template below */

/*-------------------------------------------------------------------------------------------------
* Specific requests are made up a project-id, workplan-type, workplan-id, unique-dpid, version-id           
*--------------------------------------------------------------------------------------------------
*
*        If this is your request   . . . . .  then set parameter values as follows 
*   ---------------------------------     -------------------------------------------------------
*   [Task Order 7 - NME Dabigatran]       %let MSProjID = to07_nme_dabi ;  
*   [WorkPlan Type]                       %let MSWPType = ahr; 
*   [Workplan # 5]                        %let MSWPID = wp005 ; 
*   [Unique DP id]                        %let MSDPID = nsdp ;     
*   [Beta 3]                              %let MSVerID = b03 ;         
*   [use underscores as delimiter]        %let dlm = _ ;                
*
*   The example values above would produce a Request-Id equal to to07_nme_dabi_ahr_wp005_nsdp_b03
*
*--------------------------------------------------------------------------------------------------*/

* Specify project-id, workplan-type, worplan-id, and version-id *;
   /* Uncomment before sending to sites. The tags below are assigned by SOC*/
/*
%LET MSProjID = opi ;
%LET MSWPType = ahr;
%LET MSWPID = wp001 ;
%LET MSDPID = nsdp ;
%LET MSVerID = b01 ;
 
 %let dlm = _ ; 
 

* Request-Id *. 
 * MSReqID is expected by standard MSOC code below.;

 *%let MSReqID = &MSProjID.&dlm.&MSWPType.&dlm.&MSWPID.&dlm.&MSDPID.&dlm.&MSVerID ;
* Delete macro variable used as delimiter as we no longer need it *;
%symdel dlm ; 
  */

 /*	 Do not edit between this line and the line  "2- call macros". */

/* Request-Id/Workplan parameter value must be lower case */
%let MSReqID = %lowcase(&MSReqID) ;
*-------------------------------------------------------------------------------------------------
* 0- Setup Environment
*-------------------------------------------------------------------------------------------------;

/*---------------------------------------------------------------------*/
/* NOTE: This is standard environment setup code for distributed regression. -- Do Not Edit   */
/*---------------------------------------------------------------------*/

/* System options */

options obs=MAX ;
options nosyntaxcheck ;
options merror serror ;
options dkricond=error dkrocond=error ;
options dsoptions=nonote2err noquotelenmax ;
options reuse=no ;
options mergenoby= nowarn ;

OPTIONS NOXWAIT NOXSYNC;

/*------------------------------------------------------------------------------------------------
* NOTES:
*   For reasons of compatibility and standardization, paths must meet the following criteria:
*     1- Starting with the workplan subdirectory, all subdirectory names must be specified in
*        lower case (for UNIX compatibility)
*     2- Slashes must be forward-leaning slashes '/' (for UNIX compatibility)
*     3- Paths must end with a final forward-leaning slash '/' (extra slashes are fine)
*------------------------------------------------------------------------------------------------*/

  %LET central_site_prg_root=&sites_prg_root_dp.;

/* Derive full path for standard Sentinel folders */
%let DPLOCAL = &sites_prg_root_dp.&MSReqID./dplocal/ ; 
%let MSOC = &sites_prg_root_dp.&MSReqID./msoc/ ;            
%let INFOLDER = &sites_prg_root_dp.&MSReqID./inputfiles/ ;  
%let SASPROGRAMS = &sites_prg_root_dp.&MSReqID./sasprograms/ ;
%let SASMACR=&Infolder.macros/;

/* Assign libnames */
LIBNAME dplocal "&dplocal";
LIBNAME msoc "&msoc.";
LIBNAME infolder "&infolder.";

**************************************************************************************************
*******                                  BEGIN PROGRAM                                     *******
**************************************************************************************************;

/*---------------------------------------------------------------------*/
/* NOTE: Programmer preparing Workplan Request-ID includes/writes      */
/*       the rest of the program below                                 */
/*---------------------------------------------------------------------*/


  * Set up log file*;

  %LET prg_name_sh=&MSReqID.;

  %LET log_file=&msoc.&prg_name_sh..log;
  %LET lst_file=&msoc.&prg_name_sh..lst;
      /*For DP*/
  * %LET lst_file=&dplocal.&prg_name_sh..lst;
 %PUT "&log_file.";
  %PUT "&lst_file.";

 * Set up log and list file*;
  /*
 proc printto log= "&log_file." new PRINT="&lst_file." new;;
run ;
 */

* Include macros ;
  	/* General utility macros for DP's and central */
   %include "&sasmacr.ms_macros_v2.sas";
  %include "&sasmacr.ms_time.sas";
  %include "&sasmacr.file_watcher.sas"; 
    %include "&sasmacr.check_var_list_in_tbl.sas";
    %include "&sasmacr.create_empty_file.sas"; 
   %include  "&sasmacr.delete_ext_file.sas" ;
   %include  "&sasmacr.delete_sas_tables.sas" ; 
   %include "&sasmacr.output_file_list.sas";
   %include "&sasmacr.find_files_mod_dtm.sas"; 
   %include "&sasmacr.calculate_sscp.sas";
   %include "&sasmacr.sum_by_pct2.sas";  
   %include "&sasmacr.export_sas_to_dlm_txt.sas";  
   %include "&sasmacr.import_dlm_txt_to_sas.sas";  
    %include "&sasmacr.modify_colnames.sas"; 
  * %include "&sasmacr.chk_dir.sas";
   	%include "&sasmacr.chk_create_dir.sas";	* Uses SAS functions instead of  shell command;
   
   					/*Central only utility macros */
   %include "&sasmacr.calc_matrix_product.sas";
   %include "&sasmacr.solve_linear_equations_reg.sas";
   %include "&sasmacr.lin_reg_with_sscp_input.sas";
   %include "&sasmacr.create_dir_structure_central.sas"; 

				/*Distributed regression specific  central*/
	%include "&sasmacr.distributed_regression.sas"; 
					  /*Both Genmod and Cox */
    %include "&sasmacr.d_reg_central_step1_vglc.sas"; 
    %include "&sasmacr.d_reg_central_step2_vglc.sas";
	%include "&sasmacr.create_parameters_tbl.sas";
    %include "&sasmacr.watch_dp_msoc.sas";   
    %include "&sasmacr.combine_dp_tbls.sas";
    %include "&sasmacr.cleanup_dp_msoc.sas"; 
    %include "&sasmacr.check_track_convergence_vglc.sas"; 

	 /*Genmod regression only (linear and logistic) macros*/
   %include "&sasmacr.calc_genmod_iter_stats_central_v2.sas"; 
   %include "&sasmacr.calc_hl_stats_central.sas"; 
   %include "&sasmacr.calc_hl_stats.sas"; 
	%include "&sasmacr.calc_roc.sas";
   %include "&sasmacr.calc_final_stats_genmod_central.sas";

     /*Cox regression only  macros*/
   %include "&sasmacr.calc_cox_iter_stats_central_vstr2_ef2.sas"; 
   %include "&sasmacr.calc_cox_grad_hessian.sas";
   %include "&sasmacr.calc_final_stats_cox_central_vstr2_ef2.sas";
   
     /* Macros for initializing and updating of tracking tables */
  %include "&sasmacr.initialize_tracking_tbl.sas";   
   %include "&sasmacr.update_tracking_tbl.sas"; 
   %include "&sasmacr.create_tracking_tbl_fl_lev_updt.sas"; 
   %include "&sasmacr.update_tracking_tbl_fl_lev.sas";    
 
    /*Used in testing enviroment only when parameter  test_env_cd=1*/
   %include "&sasmacr._t_loop_through_dp.sas"; 
  
        /* Macros for generating reports */
    %include "&sasmacr.create_greg_rpt.sas"; 
	%include "&sasmacr.create_cox_reg_rpt.sas"; 
	%include "&sasmacr.create_status_time_rpt.sas";




	*--------------------------------------------------------------------------------------------------
* 2- call macros
*--------------------------------------------------------------------------------------------------;

	/*The macro%create_dir_structure_central create msocN subfolders for DPs listed in dp_cd_list (if they don't exist). 
              The value of N is determined by the actual value of the dp_cd, and NOT by its position in the list. 
              The macro also define global macro variables &msocN which store the full path for these directories and
              assigns SAS libraries msocN associated with above directories. 
	  */

	  
	   %LET cntr_site_request_root=&central_site_prg_root.&MSReqID./;
		 %create_dir_structure_central(dp_cd_list=&dp_cd_list.,request_root=&cntr_site_request_root.);

	
		               * Run linear distributed regression;

				*	%LET RunID=dr1;
					%LET reg_ds_in=LINEAR_KARR_2005;
					%LET  dependent_vars=medv;
					%LET independent_vars=crim indus dis dummy_dp_var2 dummy_dp_var3;
					%LET wait_time_max=30;
					%LET wait_time_max=3600;

					%PUT  RunID=&RunID.;


    	 *------------------------Distributed Linear------------------------------------------------------------------------;

	%distributed_regression(RunID=dr1
											   ,reg_ds_in=&reg_ds_in. 
											   ,dependent_vars=&dependent_vars.
											   ,independent_vars=&independent_vars.
											   ,dp_cd_list=&dp_cd_list.
											   ,NOINT=
											   ,freq=
											   ,weight=
											   ,regr_type_cd=1
											   ,alpha=0.05
											   ,groups=10
											   ,wait_time_min=1
	    									   ,wait_time_max=&wait_time_max.
											   ,test_env_cd=0
											  ,last_runid_in =1
												) 
                                  ;

			 * Create printed report using output from DR in the msoc library. The report is written to folder ;

			 %create_greg_rpt( lib_in=msoc
                                   ,rpt_dir_out=&msoc.
                                   ,prefix_r=&RunID._
                                   ,regr_type_cd=1
                                   ,distributed_reg_in=1
								    ,rpt_format=rtf
								     );
		
*------------------------Distributed Logistic  min_count_per_grp_glob=6 ------------------------------------------------------------------------; 
  /*
					%LET RunID=dl16;
					%LET tbl_intial_est=model_coeff_0 ;
			 
					%LET reg_ds_in=LINEAR_KARR_2005;
					%LET  dependent_vars=medv_high_flag;
					%LET independent_vars=crim indus dis dummy_dp_var2 dummy_dp_var3;
					%LET wait_time_max=60;
				*	%LET wait_time_max=3600;

					%PUT  RunID=&RunID.;

	 					 %distributed_regression(RunID=&RunID.
											   ,reg_ds_in=&reg_ds_in. 
											   ,dependent_vars=&dependent_vars.
											   ,independent_vars=&independent_vars.
											   ,dp_cd_list=&dp_cd_list.
											   ,NOINT=
											   ,freq=FREQ1
											   ,weight=
											   ,regr_type_cd=2
											   ,tbl_intial_est=&tbl_intial_est.
											   ,alpha=0.05
											  ,XCONV=1e-4
											   ,groups=10
											   ,wait_time_min=1
	    									   ,wait_time_max=&wait_time_max.
											   ,max_iter_nb=15
											   ,test_env_cd=0
											  ,last_runid_in =1
											  ,min_count_per_grp_glob=6
												) 
                                      ;
*/
	
*------------------------Distributed Cox  Regression ------------------------------------------------------------------------; 

					* No stratification variables;
   /*
					%LET reg_ds_in=RECID_DR;
					%LET dp_cd_list=1 2 3;
					%LET dependent_vars=week;
					%LET independent_vars=fin age  prio;
					%LET censoring_var=arrest;
					%LET censoring_lev=0;
				    %LET ties=BRESLOW;
				* 	%LET ties=EFRON;
		
				    %LET RunID=dc1;
					%LET strata_vars=;
					 %LET tbl_intial_est=cox_model_coeff_ccs1_3  ;
				*	 %LET tbl_intial_est=cox_model_coeff_ccse1_3  ; * For ties=Efron;
					%LET tbl_events_time_set=recid_events_time_set;
				
					  %LET xconv=1e-4;
					  %LET alpha=0.05;
					  %LET freq=;
					  %LET weight=;
                    
					%PUT  RunID=&RunID.;

					%distributed_regression(RunID=&RunID.
					                                           ,dp_cd_list=&dp_cd_list.
															   ,reg_ds_in=&reg_ds_in. 
															   ,dependent_vars=&dependent_vars.
															   ,independent_vars=&independent_vars.
															   ,censoring_var=&censoring_var.
															   ,censoring_lev=0
															   ,strata_vars=&strata_vars.
															   ,ties=&ties.
															   ,NOINT=
															   ,freq=&freq.
															   ,weight=&weight.
															   ,regr_type_cd=10
															   ,tbl_intial_est=&tbl_intial_est.
															   ,tbl_events_time_set=&tbl_events_time_set.
															   ,alpha=0.05
															   ,groups=10
															   ,wait_time_min=3
					    									  ,wait_time_max=30
															  ,last_runid_in =1
															   ,test_env_cd=0
															  ,max_iter_nb=10
															  ,XCONV=&xconv.
															  ,max_numb_of_grp=10000
															   ,min_count_per_grp_glob=6
																) ;

		 %LET prefix_c=&RunID._; 
		   %PUT prefix_c=&prefix_c.;

	   %create_cox_reg_rpt( lib_in=msoc
                                            ,rpt_dir_out=&dplocal.
											,strata_vars=&strata_vars.
                                             ,prefix_r=&prefix_c.
                                            ,distributed_reg_in=1
									        ,pdf_style=DEFAULT
									        ,format_N=D12.4
                                             );
  */

   /*
				*  Cox regression with stratification variable dp_cd;

					%LET reg_ds_in=RECID_DR;
					%LET dp_cd_list=1 2 3;
					%LET dependent_vars=week;
					%LET independent_vars=fin age  prio;
					%LET censoring_var=arrest;
					%LET censoring_lev=0;
				    %LET ties=BRESLOW;
				* 	%LET ties=EFRON;
		
					  %LET RunID=dc3;
					  %LET strata_vars=dp_cd;
					  %LET tbl_intial_est=Cox_Model_Coeff_ccs3_3;
				*	 %LET tbl_intial_est=Cox_Model_Coeff_ccse3_3;   * For ties=Efron;
					  %LET tbl_events_time_set=;

					  %LET xconv=1e-4;
					  %LET alpha=0.05;
					  %LET freq=;
					  %LET weight=;
                    *  %LET weight=weight2;

					%PUT  RunID=&RunID.;

					%distributed_regression(RunID=&RunID.
					                                           ,dp_cd_list=&dp_cd_list.
															   ,reg_ds_in=&reg_ds_in. 
															   ,dependent_vars=&dependent_vars.
															   ,independent_vars=&independent_vars.
															   ,censoring_var=&censoring_var.
															   ,censoring_lev=0
															   ,strata_vars=&strata_vars.
															   ,ties=&ties.
															   ,NOINT=
															   ,freq=&freq.
															   ,weight=&weight.
															   ,regr_type_cd=10
															   ,tbl_intial_est=&tbl_intial_est.
															   ,tbl_events_time_set=&tbl_events_time_set.
															   ,alpha=0.05
															   ,groups=10
															   ,wait_time_min=3
					    									  ,wait_time_max=30
															  ,last_runid_in =1
															   ,test_env_cd=0
															  ,max_iter_nb=10
															  ,XCONV=&xconv.
															  ,max_numb_of_grp=10000
															   ,min_count_per_grp_glob=6
																) ;

		   %LET prefix_c=&RunID._; 
		   %PUT prefix_c=&prefix_c.;

	   %create_cox_reg_rpt( lib_in=msoc
                                            ,rpt_dir_out=&dplocal.
											,strata_vars=&strata_vars.
                                             ,prefix_r=&prefix_c.
                                            ,distributed_reg_in=1
									        ,pdf_style=DEFAULT
									        ,format_N=D12.4
                                             );
		*/

/*    End  main wrapper  for the  central package */