 /*HEADER-----------------------------------------------------------------------
|
| PROGRAM:  sum_by_pct2 (macro)
|
| PURPOSE:  
               The macro first ranks the &x_var and  put them into backets of approximately equal size (percentiles). 
             The number of backets can be specified by parameters &groups.
			 For example, for groups=10 the macro will attempt to backet values of x_var into ten percentiles.
               When parameter &freq and &min_count_per_grp are not specified and &fast_method_flag=1 
              the macro uses PROC RANK with TIES=MEAN
            to rank the data. Otherwise, it uses custom code to account for these parameters. 
			(PROC RANK does not have FREQ statement). The logic for treating tied values is the same as the one
            used by PROC LOGISTIC in the goodnes of fit test (Hosmer-Lemeshow test).
            The results of grouping by rank are outputed into the table &tbl_name_rank_out.

               When parameter &tbl_name_out is specified the macro also calculates mean, median and
            other descriptive statistics for each backet.
			The naming convention for output statistic follows autoname option in PROC MEANS.
			For example, for variable INCOME one will INCOME_Mean, INCOME_Median, INCOME_Std and so on.
			The macro is often used together with PROG GPLOT (SGPLOT) to visualize the relation between variables.
			The usage exampe illustrates how to use it for residual analysis of the regression output.

            At least one of the output tables: tbl_name_out,  tbl_name_rank_out has to be specified.
|
| INPUT:    MACRO PARMS:
|
|             tbl_name_in:      Input table. 
|             by_var:    		List of by variables. As usual,  statistics are calculated for
								     each group separetly. Optional. If specified the input dataset has to be
                                    sorted by &by_var.
			   
			  freq				  Has the same meaning as FREQ statement in SAS procedures.
			  WEIGHT:			Weigh variable used by in the calculations of y_var stats. It is not used in ranking of x_var. 
|             x_var:     		Independent variable
|			  y_var:     		Dependent variable
			  groups:    		Number of percentiles that x_var are grouped. Default is 10

     min_count_per_grp: Minimum number of &x_var. values per group. If parameter freq is specified 
                                      then all counts use this parameter. The parameter is particular useful when
                                      one need to satisfy HIPAA requierement in which case it usually set to 6. 

     stats_for_y_vars: Optional. Defaults to: %STR(MEAN= MEDIAN=  STD= STDERR=). 
                                  All stats available in PROC MEANS output statement can be spesified
                                  with the similar syntax. See  description tbl_name_out  for details on
                                  output variables' naming convention.

| OUTPUT:    tbl_name_out:   Optional.  Output table with with various statistics for y_var.
							  The naming convention for output statistic follows autoname option in PROC MEANS.
							  For example, for variable INCOME one will INCOME_Mean, INCOME_Median, INCOME_Std 
							  and so on. If  the value is not specified then the macro only ranks the &xvar variable
                              and put into groups. 
                      
              tbl_name_rank_out:  Optional. Contains &x_var, Rank_&x_var 
                               The Rank_&x_var  identifies the group (percentile group). The numbering starts with 0. The maximum 
                                possible value of this variable is %eval(&groups. -1) but it can be smaller if:

                                a) Number of distinct values of &x_var. is smaller than &groups.
                                b) The value of &min_count_per_grp cannot be satisfied with number of groups equal &groups.
                      

|USAGE:  %sum_by_pct2(tbl_name_in=regr_output,
			tbl_name_out= MDL_TMPY.P_UTILIZER_SUM_BY_PCT, 
			x_var=P_utilizer,y_var=utilizer RES_utilizer STDRES_utilizer,groups=100);

   

%sum_by_pct2(tbl_name_in=DPLocal.&prefix0_dp.resid_detail
			                    ,tbl_name_out= &prefix_dp.resid_sum_by_pct
			                    ,x_var= _MEAN_
                                ,y_var=_RESP_  _RESIDUAL_ _PRESIDUAL_
                                ,groups=10);
	
	%sum_by_pct2(tbl_name_in=DPLocal.&prefix0_dp.resid_detail
			                    ,tbl_name_out= &msoc_lib..&runid._resid_sum_by_pct
								,freq=_FREQ_
			                    ,x_var= _MEAN_
                                ,y_var=_RESP_  _RESIDUAL_ _PRESIDUAL_
								,min_count_per_grp=6
                                ,groups=10);


symbol1 value=none height=0.5 cv=black ci=black width=3 interpol=join ;
symbol2 value=dot height=0.5 cv=blue ci=blue width=3 co=white interpol=none;


PROC GPLOT DATA=MDL_TMPY.P_UTILIZER_SUM_BY_PCT;
     PLOT P_utilizer_Mean*P_utilizer_Mean=1 utilizer_Mean*P_utilizer_Mean=2 /overlay ;
RUN;
     PLOT RES_utilizer_Var*P_utilizer_Mean=2 ;
RUN;
      PLOT STDRES_utilizer_Var*P_utilizer_Mean=2 /vref=1;
RUN;
    QUIT;
+-------------------------------------------------------------------------------
| HISTORY:  04/11/2016 - Yury  Vilk.
|           
+-----------------------------------------------------------------------HEADER*/

 %MACRO sum_by_pct2(tbl_name_in=, tbl_name_out=, tbl_name_rank_out=
		                                    ,by_var=
                                            ,freq=
                                            ,weight=
		                                    ,x_var=,y_var=
                                            ,groups=10
                                            ,min_count_per_grp=
		                                    ,stats_for_y_vars=
										     ,fast_method_flag=0
                                           );



%GLOBAL  DEBUG_FLAG DEBUG_FLAG2;

%LOCAL by_var1 fst_by_var lst_by_var  FREQ1 WEIGHT1 freq2 
              Rank_x_var del_rank_out str_dist_cnt;

  %PUT;
  %PUT -----   Start of the macro sum_by_pct2;
  %PUT;

/*
%LET  prefix_dp=r1_i2_dp1_ ;
%LET msoc_lib=msoc1;

%LET tbl_name_in=&prefix_dp.genmod_vars;
*%LET tbl_name_in=&prefix_dp.genmod_vars_t;
 *%LET tbl_name_out= &msoc_lib..&runid._resid_sum_by_pct;
%LET tbl_name_out= &prefix_dp.resid_sum_by_pct;
%LET freq=_FREQ_  ;
%LET weight=;
%LET by_var=;
* %LET by_var=dummy_by_var ;
%LET x_var= _MEAN_;
 %LET y_var=_RESP_ _ADJ_VARIANCE_  _RESIDUAL_ _PRESIDUAL_ ;
  %LET groups=10;
  %LET min_count_per_grp=&lowest_count.;
   */
  
  %LET Rank_x_var=Rank_&x_var.;


%IF %LENGTH(&tbl_name_rank_out.)=0  %THEN 
                            %DO;
				%LET del_rank_out=__rank_out;
				%LET tbl_name_rank_out=__rank_out;
							%END;
%ELSE 	 %LET del_rank_out=;

%IF  %LENGTH(&freq.) >0  %THEN 	%DO; %LET   freq1=FREQ &freq.; %LET freq2=%STR(AND &freq. NE .) ;%END;
 %ELSE				  			                     %DO;   %LET   freq1=;   %LET   freq2=; ;%END;
	
%IF &weight NE  %THEN %DO; %LET weight1=WEIGHT &weight.; %END;
%ELSE				  %DO;  %LET weight1=; 				%END;

 %IF %LENGTH(&stats_for_y_vars.)=0 
%THEN  %LET stats_for_y_vars=%STR(MEAN= MEDIAN=  STD= );


%IF %LENGTH(&by_var.)=0 %THEN 
					%DO; 
			%LET by_var1=; 
             %LET fst_by_var=_N_;
             %LET lst_by_var=_last_;	
					%END;
%ELSE			   
					%DO; 
			%LET by_var1=BY &by_var;
			 %LET lst_var_in_by_grp=%SCAN(&by_var.,-1, %STR( ))	;
			  
			 %LET fst_by_var=FIRST.&lst_var_in_by_grp.;
             %LET lst_by_var=LAST.&lst_var_in_by_grp.; 	  
					%END;

PROC DATASETS NOWARN NOLIST;
   DELETE  &tbl_name_rank_out.;
RUN;

  %IF  %LENGTH(&freq.) =0	AND %LENGTH(&min_count_per_grp.) =0 
          AND &fast_method_flag.=1 
 %THEN
                                     %DO;
 PROC RANK DATA=&tbl_name_in(KEEP=&by_var &freq. &weight. &x_var. &y_var.) 
		   OUT= &tbl_name_rank_out. TIES=MEAN GROUPS=&groups.    PRESERVERAWBYVALUES;
 &by_var1.;
  VAR &x_var. ;
 RANKS &Rank_x_var. ;
 WHERE 	&x_var. NE . &freq2.;
 RUN;
 QUIT;

 PROC SORT DATA=&tbl_name_rank_out. ;
	BY &by_var. &Rank_x_var.;
 RUN;

 %LET str_dist_cnt=;
									  %END;	/* End of do-group for fast_method_flag=1 */
	%ELSE
									  %DO;
   %IF %LENGTH(&min_count_per_grp.) =0 %THEN %LET min_count_per_grp=6;
 
PROC DATASETS NOWARN NOLIST;
   DELETE _tmp_srt  _tmp_sum_by_xvar _tmp_sum _tmp_sum_by_xvar2 __rank_out0 ;
RUN;
QUIT;

 PROC SORT DATA=&tbl_name_in(KEEP=&by_var &freq. &weight. &x_var. &y_var.)  
                       OUT=_tmp_srt;
  BY &by_var.  &x_var.;
   WHERE 	&x_var. NE . &freq2.;
 RUN;
 QUIT;

 /* Round &x_var. to 12 significant digits to avoid problems with finite machine accuracy when comparing very close numbers*/
 DATA  _tmp_srt;
       SET _tmp_srt;

	    IF  &x_var.=0					   THEN  &x_var. = 1E-12	;
	  ELSE IF ABS(&x_var.) < 1E-12 THEN  &x_var. =SIGN(&x_var.) * 1E-12	;

	  IF INT(&x_var.) ne 0 THEN DO;
    &x_var.=ROUND(&x_var.,10**(INT(LOG10(ABS(&x_var.)))-11));
    END;
  ELSE DO;
    &x_var.=ROUND(&x_var.,10**(-1*(ABS(INT(LOG10(ABS(&x_var.))))+12)));
    END;
 RUN;

  PROC MEANS DATA=_tmp_srt NOPRINT NWAY;
   BY &by_var.  &x_var.;
  VAR  &x_var.;
  &freq1. ;
   OUTPUT OUT=_tmp_sum_by_xvar(DROP=_TYPE_ DROP=_N_) 
        	N(&x_var.)=_N_ ;
 RUN;
 QUIT;

 
  PROC MEANS DATA=_tmp_sum_by_xvar NOPRINT NWAY;
   BY &by_var. ;
  VAR  &x_var.;
  FREQ _FREQ_ ;
   OUTPUT OUT=_tmp_sum(DROP=_TYPE_ _N_ RENAME=(_FREQ_=N_total1)) 
        	N(&x_var.)=_N_ ;
 RUN;
 QUIT;

 DATA _tmp_sum_by_xvar2;
  MERGE _tmp_sum_by_xvar 
              _tmp_sum;
		IF N_total1 NE . THEN N_total=N_total1;
   RETAIN  N_total;
  &by_var1. ;

  /* Below we are using logic used in SAS Hosmer-Lemeshow Goodness-of-Fit Test*/
    target_obs_per_grp=MAX(INT(N_total/&groups. +0.5), &min_count_per_grp.);

  DROP N_total1;
 RUN;

  DATA __rank_out0;
   LENGTH &Rank_x_var. 8;
    SET _tmp_sum_by_xvar2 END=_last_;
	   BY &by_var.  &x_var.;
	     RETAIN &Rank_x_var. __cum_cnt_grp __cum_cnt __cum_dst_cnt_grp;
	   IF &fst_by_var.=1 THEN  
	   				   DO;
                 &Rank_x_var. =0;
				 __cum_dst_cnt_grp=1;
				 __cum_cnt_grp=_FREQ_;
				 __cum_cnt=_FREQ_;
				       END;
		ELSE
					   				DO;
		   __cum_cnt=__cum_cnt+_FREQ_;

		  IF	(         __cum_cnt_grp <  target_obs_per_grp 
		       		AND  __cum_cnt_grp+INT(_FREQ_/2) <=  target_obs_per_grp
			 	)
			OR ((&lst_by_var.=1 OR &Rank_x_var. =&groups.-1) AND _FREQ_ <=target_obs_per_grp/2)
		  THEN 
		              DO;
				__cum_dst_cnt_grp=__cum_dst_cnt_grp+1;
                  __cum_cnt_grp=__cum_cnt_grp+_FREQ_;
				      END;
		   ELSE  
						DO;
					/* Start new rank group */
				 &Rank_x_var.=&Rank_x_var.+1;
				 __cum_dst_cnt_grp=1;
				 __cum_cnt_grp=_FREQ_;
						END;
					   				END;

	RUN;

	DATA  &tbl_name_rank_out.;
	   LENGTH &Rank_x_var. 8;
		MERGE _tmp_srt 
                      __rank_out0(KEEP=&by_var.  &x_var. &Rank_x_var. __cum_dst_cnt_grp) ;
		  BY &by_var.  &x_var.;
	 RUN;

	 %LET str_dist_cnt=%STR(MAX(__cum_dst_cnt_grp) =&x_var._Dist_Cnt);

									   %END; /* NOT (%LENGTH(&freq.) =0	OR %LENGTH(&min_count_per_grp.) =0) */

   %IF %LENGTH(&tbl_name_out.) >0 	%THEN
                                          %DO;
  PROC MEANS DATA=&tbl_name_rank_out. NOPRINT NWAY;
   BY &by_var. Rank_&x_var. ;
  VAR  &y_var.;
  &freq1. ;
  &weight1.;
   OUTPUT OUT=&tbl_name_out(DROP=_TYPE_) 
         MIN(&x_var)=&x_var._Min MAX(&x_var)=&x_var._Max
         MEAN(&x_var)=&x_var._Mean
		 &str_dist_cnt.
		 &stats_for_y_vars.

       /*   MAX(__cum_dst_cnt_grp) =&x_var._Dist_Cnt
        MEDIAN(&x_var)=&x_var._MEDIAN
         VAR(&x_var)=&x_var._VAR  */
		
	/AUTONAME AUTOLABEL ;
 RUN;
 QUIT;
									%END;
		%IF &DEBUG_FLAG2. NE Y %THEN 
                                                     %DO;
		
PROC DATASETS NOWARN NOLIST;
  DELETE _tmp_srt  _tmp_sum_by_xvar _tmp_sum _tmp_sum_by_xvar2 __rank_out0 &del_rank_out.;
RUN;
QUIT;
		                                              %END;
  %PUT;
  %PUT -----   End of the macro sum_by_pct2;
  %PUT;

 %MEND;
