  /***************************************************************************************************
*                                           SENTINEL PROGRAM
****************************************************************************************************
* NAME:  run_d_reg_dp
*
* PURPOSE: data partner master program for distributed regression;
*
* MAJOR STEPS: 
			  INCLUDE:

 	/* General utility macros: 

   %include "&sasmacr.ms_macros.sas"; 
  %include "&sasmacr.ms_time.sas";
  %include "&sasmacr.file_watcher.sas"; 
    %include "&sasmacr.check_var_list_in_tbl.sas";
    %include "&sasmacr.create_empty_file.sas"; 
   %include  "&sasmacr.delete_ext_file.sas" ;
   %include  "&sasmacr.delete_sas_tables.sas" ; 
   %include "&sasmacr.output_file_list.sas";
   %include "&sasmacr.find_files_mod_dtm.sas"; 
   %include "&sasmacr.calculate_sscp.sas";
   %include "&sasmacr.sum_by_pct2.sas";  
   %include "&sasmacr.export_sas_to_dlm_txt.sas";  
   %include "&sasmacr.import_dlm_txt_to_sas.sas"; 

	  Distributed regression specific macros for DP:
 
   %include "&sasmacr.dp_main.sas"; 
   %include "&sasmacr.calc_genmod_vars.sas"; 
    %include "&sasmacr.calc_residual_stats_dp.sas";
	%include "&sasmacr.initialize_tracking_tbl.sas"; 
	%include "&sasmacr.create_tracking_tbl_fl_lev_updt.sas"; 
*/
* 
* KEY DEPENDENCIES/CONSTRAINTS:
*         - Common Components installation
*--------------------------------------------------------------------------------------------------
* CONTACT INFO: 
*  Mini-Sentinel Coordinating Center
*  info@mini-sentinel.org
*
*--------------------------------------------------------------------------------------------------
* HISTORY:
*  Create date (mm/dd/yy): 08/20/16
*  Last modified date (mm/dd/yy): 08/20/16
*  Version:
*
*  CHANGE LOG: 
*
*   Version   Date       Comment (reference external documentation when available)
*   -------   --------   -------------------------------------------------------------------------
*     1.1.1    08/31/16   First production version release
*
**************************************************************************************************/

/* System options */
options nosymbolgen nomlogic ;
options ls=100 nocenter ;

/*-------------------------------------------------------------------------------------------------
*                                        DEFINITIONS
*--------------------------------------------------------------------------------------------------

* MSReqID  ... Mini-Sentinel Project Request-Id
*              NOTE: This parameter needs to be set by the programmer preparing the Request-ID
*                    in coordination with the Mini-Sentinel Operations Center (MSOC)
*
* The following paths for the current request are automatically derived 
*    DPLOCAL  ... path for data created that remains local (i.e behind the DP firewall)
*    MSOC     ... path for data created to share with MS Operations Center
*    INFOLDER ... path for program inputs (e.g.lookup data, external macros)
*    SASPROGRAMS ... path for all SAS programs that are part of the data request
*-------------------------------------------------------------------------------------------------*/

 %LET min_count_per_grp=6;

 %LET dp_cd=1;
 * %LET sysparm= ;

 DATA _NULL_ ;
  LENGTH sysparm $ 3000 ; 
  IF SYMEXIST("sysparm") THEN sysparm=DEQUOTE(STRIP(SYMGET("sysparm")));

  IF sysparm NE " " THEN CALL SYMPUT("dp_cd",STRIP(sysparm)); 
  PUT _ALL_ ;
 RUN;

%PUT dp_cd=&dp_cd.;

* %INCLUDE "J:\prshare\MSCDM\PRG\Mod_Prg_Physical_MSCDM\_Dev\Distributed_Regression\ver1\inputfiles\macros\parms_file.sas";

%LET sites_prg_root=%STR(F:\ASPE\Publications\Statistical Software Manuscripts\Testing Package 2019\Test 2\);
%INCLUDE "&sites_prg_root.parms_file2.sas";

			   * END OF  HPHC TESTING/DEVELOPMENT SECTION ONLY;

	/* Request-Id/Workplan parameter value must be lower case */



**************************************************************************************************
*******                                  END OF USER INPUT                                  ******
*******                             DO NOT EDIT BELOW THIS LINE                             ******
*******                        (Consult with MSOC or MS team leader                         ******
*******                           if you feel edits are required)                           ******
**************************************************************************************************;

/*---------------------------------------------------------------------*/
/* NOTE: Programmer preparing Request-Id defines the                   */
/*       parameter MSReqID in coordination with the MSOC               */
/*---------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------------------------
* Specific requests are made up a project-id, workplan-type, workplan-id, unique-dpid, version-id           
*--------------------------------------------------------------------------------------------------
*
*        If this is your request   . . . . .  then set parameter values as follows 
*   ---------------------------------     -------------------------------------------------------
*   [Task Order 7 - NME Dabigatran]       %let MSProjID = to07_nme_dabi ;  
*   [WorkPlan Type]                       %let MSWPType = ahr; 
*   [Workplan # 5]                        %let MSWPID = wp005 ; 
*   [Unique DP id]                        %let MSDPID = nsdp ;     
*   [Beta 3]                              %let MSVerID = b03 ;         
*   [use underscores as delimiter]        %let dlm = _ ;                
*
*   The example values above would produce a Request-Id equal to to07_nme_dabi_ahr_wp005_nsdp_b03
*
*--------------------------------------------------------------------------------------------------*/

* Specify project-id, workplan-type, worplan-id, and version-id *;
   /* Uncomment before sending to sites. The tags below are assigned by SOC*/
/*
%LET MSProjID = opi ;
%LET MSWPType = ahr;
%LET MSWPID = wp001 ;
%LET MSDPID = nsdp ;
%LET MSVerID = b01 ;
 
 %let dlm = _ ; 
 

* Request-Id *. 
 * MSReqID is expected by standard MSOC code below.;

 *%let MSReqID = &MSProjID.&dlm.&MSWPType.&dlm.&MSWPID.&dlm.&MSDPID.&dlm.&MSVerID ;
* Delete macro variable used as delimiter as we no longer need it *;
%symdel dlm ; 
  */


/* Request-Id/Workplan parameter value must be lower case */
%let MSReqID = %lowcase(&MSReqID) ;
*-------------------------------------------------------------------------------------------------
* 0- Setup Environment
*-------------------------------------------------------------------------------------------------;

/*---------------------------------------------------------------------*/
/* NOTE: This is standard MSOC environment setup code -- Do Not Edit   */
/*---------------------------------------------------------------------*/

/* System options */
 /*
options obs=MAX ;
options msglevel=i ;
options mprint mprintnest ;
options errorcheck=strict errors=0 ;
options merror serror ;
options dkricond=error dkrocond=error mergenoby=warn;
options dsoptions=nonote2err noquotelenmax ;
options reuse=no ;
options fullstimer ;
*/

options obs=MAX ;
options nosyntaxcheck ;
options merror serror ;
options dkricond=error dkrocond=error;
options dsoptions=nonote2err noquotelenmax ;
options reuse=no ;
options mergenoby= nowarn ;

/*------------------------------------------------------------------------------------------------
* NOTES:
*   For reasons of compatibility and standardization, paths must meet the following criteria:
*     1- Starting with the workplan subdirectory, all subdirectory names must be specified in
*        lower case (for UNIX compatibility)
*     2- Slashes must be forward-leaning slashes '/' (for UNIX compatibility)
*     3- Paths must end with a final forward-leaning slash '/' (extra slashes are fine)
*------------------------------------------------------------------------------------------------*/


/* Define Workplan Specific Subdirectories */
%let DPLOCAL = &_root_dplocal.&MSReqID./dplocal/ ; 
%let MSOC = &_root_msoc.&MSReqID./msoc/ ;            
%let INFOLDER = &_root_inputfiles.&MSReqID./inputfiles/ ;  
%let SASPROGRAMS = &_root_sasprograms.&MSReqID./sasprograms/ ;

 
*For Testing; 
*%LET SASMACR=&mod_prg_root.inputfiles/macros/; 
/*For Production*/
%let SASMACR=&infolder.macros/;

/* Assign libnames */
LIBNAME dplocal "&dplocal";
LIBNAME msoc "&msoc.";
LIBNAME infolder "&infolder.";
LIBNAME data_in "&data_in_dir.";

   %LET infolder_dir=&infolder.;
   	%LET infolder_lib=infolder;
	%LET msoc_dir=&msoc.;
	%LET msoc_lib=msoc;
	%LET dplocal_dir=&dplocal.;
   	%LET dplocal_lib=dplocal;
   	%LET data_in_lib=data_in;
	%LET exit_after_each_iter=0;


	   %PUT dp_cd=&dp_cd.;

**************************************************************************************************
*******                                  BEGIN PROGRAM                                     *******
**************************************************************************************************;

/*---------------------------------------------------------------------*/
/* NOTE: Programmer preparing Workplan Request-ID includes/writes      */
/*       the rest of the program below                                 */
/*---------------------------------------------------------------------*/


  * Set up log file*;

  %LET prg_name_sh=&MSReqID.;

  %LET log_file=&msoc.&prg_name_sh..log;
  %LET lst_file=&dplocal.&prg_name_sh..lst;
      /*For DP*/
  * %LET lst_file=&dplocal.&prg_name_sh..lst;
 %PUT "&log_file.";

  %PUT "&lst_file.";

  * Set up log and list file*;
 /*
 proc printto log= "&log_file." new PRINT="&lst_file." new;;
run ;
 */

* Include macros ;
  	/* General utility macros for DP's and central */
   %include "&sasmacr.ms_macros_v2.sas"; 
  %include "&sasmacr.ms_time.sas";
  %include "&sasmacr.file_watcher.sas"; 
    %include "&sasmacr.check_var_list_in_tbl.sas";
    %include "&sasmacr.create_empty_file.sas"; 
   %include  "&sasmacr.delete_ext_file.sas" ;
   %include  "&sasmacr.delete_sas_tables.sas" ; 
   %include "&sasmacr.output_file_list.sas";
   %include "&sasmacr.find_files_mod_dtm.sas"; 
   %include "&sasmacr.calculate_sscp.sas";
   %include "&sasmacr.sum_by_pct2.sas";  
   %include "&sasmacr.export_sas_to_dlm_txt.sas";  
   %include "&sasmacr.import_dlm_txt_to_sas.sas"; 
    %include "&sasmacr.modify_colnames.sas"; 


   	  /*Distributed regression specific macros for DP */
   %include "&sasmacr.dp_main_vglc.sas";	 
 
   %include "&sasmacr.calc_genmod_iter_stats_dp.sas"; 
   %include "&sasmacr.calc_genmod_vars.sas"; 
   %include "&sasmacr.calc_residual_stats_genmod_dp.sas";

   %include "&sasmacr.calc_cox_iter_stats_dp_vstr2_ef2.sas";	
    %include "&sasmacr.calc_cox_grad_hessian.sas";  
	%include "&sasmacr.calc_residual_stats_cox_dp_vstr2_ef2.sas";

	 %include "&sasmacr.initialize_tracking_tbl.sas"; 
	%include "&sasmacr.create_tracking_tbl_fl_lev_updt.sas"; 
	
*--------------------------------------------------------------------------------------------------
* 2- call macros
*--------------------------------------------------------------------------------------------------;

	   %dp_main(dp_cd=&dp_cd.);
